#include <TimerOne.h>
#include <TimerThree.h>
#include <Wire.h>
#include <batteryCheck.h>
#include <inclinometer.h>
#include <motors.h>
#include <dynamixel.h>
#include <dynamixelUse.h>
#include <uSonic.h>


//#define      __DEBUG__


// Constants

#define BUFFER_SIZE      128                // Buffer size (Transmit and receive)
#define PUSH_BUTTON      10                 // ID of the pin connected to the push button
#define SERVO_ID         1                  // ID of the servomotor
#define MAX_FRAME_LENGHT 32                 // Maximum number of byte in a frame
#define ADRESS_US_1      0xE4               // I²C adress of the left sensor (left sensor)
#define ADRESS_US_2      0xEE               // I²C adress of the second sensor (center sensor)
#define ADRESS_US_3      0xF6               // I²C adress of the third sensor (third sensor)

// Functions


bool CheckCode(char *Buffer,const char *Code,int Lenght);       // Check if the string is valid and first characters are equal to the code
bool CheckCRCFrame();                                           // Return true if the CRC of the frame is OK
void ResetBufferTx();                                           // Reset transmission buffer
void AddCharBufferTx(char Val);                                 // Add an integer in the transmission buffer
void AddIntBufferTx(int Val);                                   // Add an integer in the transmission buffer
void AddLongBufferTx(long Val);                                 // Add a long in the transmission buffer
void AddFloatBufferTx(float Val);                               // Add an integer in the transmission buffer
void SendBufferTx();                                            // Finalize and send the transmission buffer



// Global variables

unsigned char      BfrRx[BUFFER_SIZE];          // Buffer for data received from the PC
unsigned char      BfrTx[BUFFER_SIZE];          // Buffer with data to send to the PC
int                IndBfrRx=0;                  // Current index of the received buffer
int                IndBfrTx=0;                  // Current index of the transmit buffer
Dynamixel          ServoLidar;                  // Create the object for controlling the servomoteur
unsigned int       StateRx=0;                   // Current state of the state machine for communication


void setup()
{
  // Initialize UARTs
  pinMode(PUSH_BUTTON, INPUT);             // For button (configured as imput)
  digitalWrite (PUSH_BUTTON,HIGH);
  Serial.begin (115200);                   // For communication with the PC
  Serial1.begin(115200);                   // For communication with the left motor
  Serial2.begin(115200);                   // For communication with the right motor
  US_Init();                               // Initialize I²C link for ultrasonic sensors

  int Err=0;                               // Error code     x|x|x|x|x|x| Right Motor | Left Motor
  
  
  // Initialize motors
  Err=Err | (!ResetLeft() << 0);           // Reset motor controllers
  Err=Err | (!ResetRight() << 1); 
  

  if (Err==0)                              // Check error number
    Serial.print("OK\n");
  else
  {
    Serial.print("ER");
    Serial.print(Err,DEC);
  }
  
  
//  Timer3.initialize(1000);                             // Initialize Timer 3 with a period of 1ms
//  Timer3.attachInterrupt(callback_INT_Timer3);         // attaches callback() as a timer overflow interrupt

}

/*************************************************/
/*       State machine for receiving data        */
/*************************************************/

void loop()
{
  // Check if a new charactere has been received from the PC
  if (Serial.available())
  {
    switch (StateRx)
    {
      

       case 0 :  BfrRx[0]=Serial.read();                                        // Test if the first 0xFF has been receiver
                 if (BfrRx[0]==0xFF) 
                 {
                   StateRx=10;
                   IndBfrRx=1;
                 }
                 break;
                 
      case 10 :  BfrRx[1]=Serial.read();                                        // Test if the second 0xFF has been receiver
                 if (BfrRx[1]==0xFF) 
                 {
                   StateRx=20;
                   IndBfrRx=2;
                 }
                 else
                 {
                   IndBfrRx=0;
                   StateRx=0;
                 }
                 break;
                 
      case 20 :  BfrRx[2]=Serial.read();                                        // Read and check the number of byte in the frame
                 if (BfrRx[2]<=MAX_FRAME_LENGHT)
                 {
                   StateRx=30;
                   IndBfrRx=3;
                 }
                 else
                 {
                   StateRx=0;
                   IndBfrRx=0;
                 }
                 break;
                 
         
      case 30 :  BfrRx[IndBfrRx++]=Serial.read();                              // Read the number of bytes awaited
                 if (IndBfrRx==BfrRx[2]+4)
                 {
                   if (CheckCRCFrame())
                   {
#ifdef __DEBUG__                     
                     Serial.println ("CRC OK");
#endif
                     RunFrame();
                   }
                   else
                   {
#ifdef __DEBUG__                     
                     Serial.println ("CRC ERROR");
#endif                     
                   }
                   StateRx=0;
                   IndBfrRx=0;
                 }                 
                 break;
                 
    } // switch

#ifdef __DEBUG__
    Serial.print ("State : ");
    Serial.println (StateRx);
#endif
  }
}
    
    
 
// This function analyse and execute the instruction included in the current frame
// Frame is assumed to be valid (CRC is correct)
void RunFrame()
{
  unsigned short NewPos;
  short          LeftSpeed,RightSpeed;
  short          Acc;
  long           LeftPos,RightPos;
  byte           US_ID;
  
  switch (BfrRx[3])
  {
    
    /**************************************/
    /*         PC Request data            */
    /**************************************/
    
    
    case 'b'  :   ResetBufferTx();                                    // b : request batteries voltage
                  AddIntBufferTx(Battery1State_mV());
                  AddIntBufferTx(Battery2State_mV());    
                  SendBufferTx();
                  break;


    case 'i'  :   ResetBufferTx();                                    // i : request inclinometer data
                  AddFloatBufferTx(GetRoll());
                  AddFloatBufferTx(GetPitch());
                  SendBufferTx();
                  break;


    case 'p'  :   ResetBufferTx();                                    // p : request push-button state
                  AddCharBufferTx(digitalRead (PUSH_BUTTON));
                  SendBufferTx();
                  break;


    case 's'  :   ResetBufferTx();                                    // s : request servomotor position
                  AddIntBufferTx(ServoLidar.GetPresentPosition(SERVO_ID));
                  SendBufferTx();
                  break;
                  
                  
    case 'm'  :   ResetBufferTx();                                    // m : request motors position
                  AddCharBufferTx(GetLeftPos(&LeftPos));
                  AddCharBufferTx(GetRightPos(&RightPos));
                  AddLongBufferTx(LeftPos);
                  AddLongBufferTx(RightPos);
                  SendBufferTx();
                  break;

    case 'u'  :   US_ID=*(byte*)(BfrRx+4);                            // U : request US mesurement
                  ResetBufferTx();
                  switch (US_ID)
                  {
                    case 1 :   AddIntBufferTx(US_GetRange(ADRESS_US_1)); break;
                    case 2 :   AddIntBufferTx(US_GetRange(ADRESS_US_2)); break;
                    case 3 :   AddIntBufferTx(US_GetRange(ADRESS_US_3)); break;
                    default :  AddIntBufferTx(-3);
                  }
                  SendBufferTx();                  
                  break;          
                  
    /**************************************/
    /*          PS send command           */
    /**************************************/




    case 'S'  :   NewPos=*(unsigned short*)(BfrRx+4);                 // S : modify servomotor position 
                  ResetBufferTx();
                  AddCharBufferTx(ServoLidar.SetGoalPosition(SERVO_ID,NewPos));
                  SendBufferTx();                  
                  break;
                  
                  
    case 'T'  :   NewPos=*(unsigned short*)(BfrRx+4);                 // T : modify servomotor speed
                  ResetBufferTx();
                  AddCharBufferTx(ServoLidar.SetMovSpeed(SERVO_ID,NewPos));
                  SendBufferTx();                  
                  break;
                  

    case 'E'  :   ResetBufferTx();                                    // E : enable tracks motors
                  AddCharBufferTx(EnableLeft());
                  AddCharBufferTx(EnableRight());
                  SendBufferTx();
                  break;
    
                  
    case 'V'  :   LeftSpeed=*(unsigned short*)(BfrRx+4);              // V : set tracks motors velocity    
                  RightSpeed=*(unsigned short*)(BfrRx+6);
                  ResetBufferTx();
                  AddCharBufferTx(SetLeftSpeed(-LeftSpeed));
                  AddCharBufferTx(SetRightSpeed(RightSpeed));
                  SendBufferTx();                  
                  break;

    case 'A'  :   Acc=*(unsigned short*)(BfrRx+4);                    // A : set tracks motors acceleration
                  ResetBufferTx();
                  AddCharBufferTx(SetLeftAcc(Acc));
                  AddCharBufferTx(SetRightAcc(Acc));
                  AddCharBufferTx(SetLeftDec(Acc));
                  AddCharBufferTx(SetRightDec(Acc)); 
                  SendBufferTx();                  
                  break;
          
    case 'U'  :   US_ID=*(byte*)(BfrRx+4);                            // U : request US mesurement
                  ResetBufferTx();
                  switch (US_ID)
                  {
                    case 1 :   AddCharBufferTx(US_RequestMesurement(ADRESS_US_1)); break;
                    case 2 :   AddCharBufferTx(US_RequestMesurement(ADRESS_US_2)); break;
                    case 3 :   AddCharBufferTx(US_RequestMesurement(ADRESS_US_3)); break;
                    default :  AddCharBufferTx(-2);
                  }
                  SendBufferTx();                  
                  break;                  
                  
    default   :   break;
       
  }
}
    
    
    
    
/**************************************/
/*        Interrupt of timer 3        */
/**************************************/
    
void callback_INT_Timer3()
{
  // Timer counter
  static int       US_State=0;                  // Current state of the state machine managing the ultrasonic sensors
  static int milliseconds=0;
  static int seconds=0;
  milliseconds++;
  if (milliseconds==1000)
  {
    seconds++;
    milliseconds=0;
  }
}    
    
    
/**************************************/
/*        Transmission functions      */
/**************************************/

    
    
    
// Check the CRC of the current frame
// Return TRUE if CRC if OK
// Return FALSE in case of error
bool CheckCRCFrame()
{
  unsigned int Sum=0;
  for (int i=2;i<3+BfrRx[2];i++)
    Sum+=BfrRx[i];   
    Sum=(~Sum) & 0xFF;

#ifdef __DEBUG__    
  Serial.println (Sum,DEC);
  Serial.println (BfrRx[IndBfrRx-1],DEC);
#endif __DEBUG__

  if ( Sum == BfrRx[IndBfrRx-1])
    return true;
  return false;
}


      
      
// Reset the transmission buffer
void ResetBufferTx()
{
  BfrTx[0]=0xFF;
  BfrTx[1]=0xFF;
  IndBfrTx=3;
}

// Add an integer in the buffer
void AddCharBufferTx(char Val)
{
  unsigned char* Pt=(unsigned char*)&Val;
  BfrTx[IndBfrTx++]=*Pt;
}

// Add an integer in the buffer
void AddIntBufferTx(int Val)
{
  unsigned char* Pt=(unsigned char*)&Val;
  BfrTx[IndBfrTx++]=*Pt;
  BfrTx[IndBfrTx++]=*(Pt+1);
}



// Add an integer in the buffer
void AddLongBufferTx(long Val)
{
  unsigned char* Pt=(unsigned char*)&Val;
  BfrTx[IndBfrTx++]=*Pt;
  BfrTx[IndBfrTx++]=*(Pt+1);
  BfrTx[IndBfrTx++]=*(Pt+2);
  BfrTx[IndBfrTx++]=*(Pt+3); 
}


// Add an integer in the buffer
void AddFloatBufferTx(float Val)
{
  unsigned char* Pt=(unsigned char*)&Val;
  BfrTx[IndBfrTx++]=*Pt;
  BfrTx[IndBfrTx++]=*(Pt+1);
  BfrTx[IndBfrTx++]=*(Pt+2);
  BfrTx[IndBfrTx++]=*(Pt+3);  
}


// Finalize and send the transmission buffer
void SendBufferTx()
{
  BfrTx[2]=IndBfrTx-3; 
 
  int Sum=0;  
  for (int i=2 ; i<BfrTx[2]+3 ; i++)
  {
    Sum+=BfrTx[i];   
//    Serial.println (Sum,DEC);
  }
  Sum=(~Sum) & 0xFF;
  BfrTx[IndBfrTx++]=Sum;
  for (int i=0;i<IndBfrTx;i++)
    Serial.print(BfrTx[i]);   
}
