/**\file dynamixel.cpp 
 * \brief Simple functions to communicate with servomotors RX-24F from <em>Robotis</em>.
 * \details See also : \n \n dynamixel.h \n The header File Reference. \n \n Dynamixel \n A class which uses the present library.*/

/* Basic methods to communicate with servomotors */ 

#include "dynamixel.h"


bool   TimeOutFlag=false; 


// Initialize communication with servomotors  
void InitDynamixel()  
{ 
#ifdef __DEBUG__DYNAMIXEL__
	Serial.println("Initialisation of the communication with servomotors");
#endif

	Serial3.begin(117647);									// Serial link connected via USART3
    pinMode(2, OUTPUT);										// Pin of Half-Duplex configured as an output
    digitalWrite(2,LOW);									// Communication in receiving mode
} 
  

// Time out of serial link in receiving mode
void ISR_TimeOut_Dynamixel()
{
    TimeOutFlag=true;										// Set the TimeOutFlag to true means that receive has failed
    Timer1.stop();											// Stop Timer1

#ifdef __DEBUG__DYNAMIXEL__
    Serial.println("TimeOut...");
#endif
}


// Method to send a packet to the servomotor which ID is specified in parameter 
// Packet Structure : 0xFF, 0xFF, ID, Length, Instruction, Parameters, Checksum
// Nb represents the number of parameters 
void SendPacket (unsigned char ID, unsigned char Instruction,unsigned char * Parameters=0,int Nb=0)
{
#ifdef __DEBUG__DYNAMIXEL__
    Serial.println("Sending packet to servomotors...");
#endif

    unsigned int Sum=0,u=0;
    while(u<Nb){											// Sum of the buffer terms
      Sum+=Parameters[u];
      u++;}
    unsigned char checksum=(~(ID+Instruction+(Nb+0x02)+Sum));// Calculation of the checksum with the Sum calculated previouly

#ifdef __DEBUG__DYNAMIXEL__
    Serial.println(checksum,HEX);
#endif

    digitalWrite(2,HIGH);									// Pin of Half-duplex in sending mode
    Serial3.write(0xFF);									// Start of frame
    Serial3.write(0xFF);									// Start of frame
    Serial3.write(ID);										// ID of the receiver
    Serial3.write(Nb+0x02);									// Length
    Serial3.write(Instruction);
    for(int n=0;n<Nb;n++)									// Buffer (instruction + param.)
      Serial3.write(Parameters[n]);
    Serial3.write(checksum);								// Checksum
      
    while (!(UCSR3A &(1<<UDRE3)));							// Wait for the last byte to be send
    delayMicroseconds(100);

    digitalWrite(2,LOW);									// Communication in receiving mode

#ifdef __DEBUG__DYNAMIXEL__
    Serial.println("Transmission complete");    
#endif
}


//Method to receive a return packet of servomotor
//The return packet is write in the buffer until Time Out is not finished
//and written data has not overflowed the NbMax data in parameter
int ReceivePacket (unsigned char * Buffer,int NbMax, long TimeOut_Us=1000000)
{
#ifdef __DEBUG__DYNAMIXEL__
    Serial.println("Reception in progress...");
	Serial.print("A lire : ");
    Serial.println(Serial3.available());
    Serial.println();
#endif

    int n=0;
    Timer1.initialize(TimeOut_Us);							// Initialize the duration of the TimeOut (in s)
    delay(100);
    
    Timer1.attachInterrupt(ISR_TimeOut_Dynamixel);			// Attach the interrupt method to the TimeOut
    TimeOutFlag=false;
    Timer1.start();											// Start of Timer1
	
	while(TimeOutFlag==false && n<NbMax && Serial3.available()>0)// Write in buffer until conditions are verified (see function description for details)
    {       
            Buffer[n] = Serial3.read();						// Read next value and write it in the buffer
#ifdef __DEBUG__DYNAMIXEL__            
			Serial.print(Buffer[n],HEX);
            Serial.print("\t");
            Serial.print("Reste : ");
            Serial.println(Serial3.available());
#endif
			n++;    
    }
    Timer1.stop();
    Timer1.detachInterrupt();

    if(n>=6&&TimeOutFlag==false){                           // If n>=6 (minimal size of a return frame)
#ifdef __DEBUG__DYNAMIXEL__
      Serial.println("\t \t \tSuccess");					// Indicate that reception is a success
#endif
      TimeOutFlag=false;									// Return to the TimeOutFlag default value
      return 1;
    }
#ifdef __DEBUG__DYNAMIXEL__
    Serial.println("\t \t \tFailure");					// Else indicate that reception is a failure
#endif
	return 0;
}


// Check if a return packet contains no error
int Errors(unsigned char * Packet)
{
#ifdef __DEBUG__DYNAMIXEL__
    Serial.println("Checking packet...");    
#endif

	if(Packet[0]!=0xFF || Packet[1]!=0xFF)					// First, check that initial frame is correct                      
	{
#ifdef __DEBUG__DYNAMIXEL__	
		Serial.println("ERROR : il manque le dbut de la trame");
#endif
		return 1;											// If initial frame incorrect, cancel operation
    }
    else
    {
       int err=0;
       if(Packet[4]&1){
#ifdef __DEBUG__DYNAMIXEL__
         Serial.println("ERROR : Input Voltage Error");
#endif
         err++;
       }
       if(Packet[4]&2){
#ifdef __DEBUG__DYNAMIXEL__
         Serial.println("ERROR : Angle Limit Error");
#endif
         err++;
       }
       if(Packet[4]&4){
#ifdef __DEBUG__DYNAMIXEL__
         Serial.println("ERROR : Overheating Error");
#endif
         err++;
       }
       if(Packet[4]&8){
#ifdef __DEBUG__DYNAMIXEL__
         Serial.println("ERROR : Range Error");
#endif		 
         err++;
       }
       if(Packet[4]&16){
#ifdef __DEBUG__DYNAMIXEL__
         Serial.println("ERROR : Checksum Error");
#endif		 
         err++;
       }
       if(Packet[4]&32){
#ifdef __DEBUG__DYNAMIXEL__
         Serial.println("ERROR : Overload Error");
#endif		 
         err++;
       }
       if(Packet[4]&64){
#ifdef __DEBUG__DYNAMIXEL__
         Serial.println("ERROR : Instruction Error");
#endif		 
         err++;
       }
       if(err!=0) return 2;									// If  errors are reported, return 2
	}
	return 0;													// If no errors return 0
}  


// Empty the reception buffer (usefull when a reception error occurenced)
void EmptyReceptionBuffer ()
{
#ifdef __DEBUG__DYNAMIXEL__
    Serial.println("Emptying reception buffer...");    
#endif

	int *Dummy,k=0;
	while(Serial3.available())								// Read while there is data available
	{
		Dummy[k]=Serial3.read();
		k++;
	}

	Serial.println("Reception buffer is now empty");
}

 