/**\file dynamixel.h 
 * \brief Simple functions to communicate with servomotors RX-24F from <em>Robotis</em>. 
 * \details For more information about frames, see the <a href=http://support.robotis.com/en/>Robotis documentation</a>
 * and go to Product Information > Dynamixel > Communication > Instruction / Status Packet. \n \n
 * dynamixel.cpp \n The C++ File Reference. \n \n Dynamixel \n A class which uses the present library. */

#ifndef dynamixel_h
#define dynamixel_h


#include <WProgram.h>
#include <TimerOne.h>


//#define __DEBUG__DYNAMIXEL__


/*
  dynamixel.h - Library for controling Dynamixel RX-24F
  from Robotis


  Copyright (c) 2011 David Drissi.  All right reserved.
  Property of the University of Angers.

  Contact : philippe.lucidarme@univ-angers.fr

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.  
*/

/*! Instruction to send a ping */
#define PING			0x01
/*! Instruction to read a data */
#define READ			0x02
/*! Equivalent to \link READ \endlink */
#define READ_DATA		0x02
/*! Instruction to write a data */
#define WRITE			0x03
/*! Equivalent to \link WRITE \endlink */
#define WRITE_DATA		0x03
/*! Instruction that works like \link WRITE \endlink but waits \link ACTION \endlink to execute */
#define REG_WRITE		0x04
/*! Instruction which executes the last \link REG_WRITE \endlink send (if any)*/
#define ACTION			0x05
/*! Instruction to return servo to factory presets */
#define RESET			0x06
/*! Instruction to command several servos simultaneously */
#define SYNC_WRITE		0x83


/*! Initialize transmission with servomotors
 * \return no return*/
void	InitDynamixel();

/*! Send a packet to all servos (if ID is defined as 0xFE) or just one (if another ID is specified)
 * \param ID Defines the servo to communicate with
 * \param Instruction The instruction to send (see the defines chapter to see other possible actions)
 * \param Parameters An array which contains the parameters of the instruction (if any)
 * \param Nb Specifies the number of parameters
 * \return no return*/
void	SendPacket (unsigned char ID, unsigned char Instruction,unsigned char * Parameters,int Nb);

/*! Receive a return packet for servomotors (a timeout is specified to prevent a reception error)
 * \param Buffer Used to save communication reception
 * \param NbMax Defines the length of data to read
 * \param TimeOut_Us Defines the length of the time out in microseconds
 * \return an int that shows if the reception has succeeded (1 means a success and 0 a failure)*/
int		ReceivePacket (unsigned char * Buffer,int NbMax, long TimeOut_Us);

/*! Check if a return packet contains no errors
 * \param Packet The return packet to check (a character's array)
 * \return an int which represents an error if any (0 means no error, 1 means error on the debut frame and 2 that there is an error on the error bit).*/
int		Errors (unsigned char * Packet);

/*! Empty the reception buffer (usefull when a reception error occurs)
 * \return no return*/
void	EmptyReceptionBuffer ();

/*! Timeout for reception mode
 * \return no return*/
void	ISR_TimeOut_Dynamixel();

#endif