/**\file dynamixelUse.cpp 
 * \brief A class to compose frames to send to servomotors.
 * \details See also : \n \n dynamixelUse.h \n The header File Reference. \n \n Dynamixel \n For the functions description.*/

/* Preparing packets and control servomotors */

#include "dynamixelUse.h"


///////////////////// General Functions ///////////////////////////


// Configure dynamixel communication
Dynamixel::Dynamixel()
{
	TimeOut=10000000;
	InitDynamixel();
}

// End communication with servomotors
Dynamixel::~Dynamixel()
{
	Serial3.end();

#ifdef __DEBUG__DYNAMIXEL_USE__
	Serial.println("Communication with servomotors close");
#endif
}

// Send a ping
bool Dynamixel::SendPing(unsigned char ID)
{
#ifdef __DEBUG__DYNAMIXEL_USE__
	Serial.print("Ping...");
#endif

	unsigned char Buffer[6];
	SendPacket(ID,PING,0,0);
	ReceivePacket(Buffer,6,TimeOut);
	
	if(Errors(Buffer))
		return 0;
	else
		return 1;
}

// Send a packet manually
bool Dynamixel::ManSend(unsigned char ID,unsigned char Instruction,unsigned char *Parameters,int NbParam,int LengthToRead)
{
#ifdef __DEBUG__DYNAMIXEL_USE__
	Serial.print("Sending packet manually...");
#endif

	unsigned char Buffer[LengthToRead+1];
	SendPacket(ID,Instruction,Parameters,NbParam);
	ReceivePacket(Buffer,LengthToRead,TimeOut);
	if(Errors(Buffer))
		return 0;
	else
		return 1;
}

// Check if packet contains no error
int Dynamixel::CheckPacket(unsigned char *Packet)
{
	return (Errors(Packet));
}

// Empty reception buffer
void Dynamixel::ClearReceptionBuffer()
{
	EmptyReceptionBuffer();
}


// Reuse servomotors after error occurenced
bool Dynamixel::LeaveAlarmShutdownMode(unsigned char ID)
{
#ifdef __DEBUG__DYNAMIXEL_USE__
	Serial.println("Leaving Alarm Shutdown mode...");
#endif
		SetTorqueLimit(ID,0);
		delay(1000);
		//bool * err=false;
		Serial.println("torque limit 0");
		SetTorqueLimit(ID,1023); //GetMaxTorque(ID,err)&& (!err)
			return 1;
	//}	
	return 0;
}

///////////////////// WRITE Functions /////////////////////////////


// Set goal position (in degrees)
bool Dynamixel::SetGoalPosition(unsigned char ID,unsigned short Angle)
{
#ifdef __DEBUG__DYNAMIXEL_USE__
	Serial.print("Setting goal position...");
#endif

	unsigned char Buffer[7];	
	unsigned char PacketGoalPos[3]={0x1E,Angle,Angle>>8};				// Prepare the packet
	SendPacket(ID,WRITE,PacketGoalPos,3);								// Send the frame
	ReceivePacket(Buffer,6,TimeOut);									// Receive the return packet

	if(Errors(Buffer))
		return 0;
	else
		return 1;
}

// Set the clockwise angle limit (in degrees)
bool Dynamixel::SetCWAngleLimit(unsigned char ID,unsigned short Angle)
{
#ifdef __DEBUG__DYNAMIXEL_USE__
	Serial.println("Setting CW angle limit...");
#endif

	unsigned char Buffer[7];	
	unsigned char PacketAngleLimit[3] ={0x06,Angle,Angle>>8};
	SendPacket(ID,WRITE,PacketAngleLimit,3);
	ReceivePacket(Buffer,6,TimeOut);

	if(Errors(Buffer))
		return 0;
	else
		return 1;
}

// Set the counter clockwise angle limit (in degrees)
bool Dynamixel::SetCCWAngleLimit(unsigned char ID,unsigned short Angle)
{
#ifdef __DEBUG__DYNAMIXEL_USE__
	Serial.println("Setting CCW angle limit...");
#endif

	unsigned char Buffer[7];	
	unsigned char PacketAngleLimit[3] ={0x08,Angle,Angle>>8};
	SendPacket(ID,WRITE,PacketAngleLimit,3);
	ReceivePacket(Buffer,6,TimeOut);

	if(Errors(Buffer))
		return 0;
	else
		return 1;
}

// Set the moving speed (0 means 0% and 0xFE means 100% of the maximum speed)
bool Dynamixel::SetMovSpeed(unsigned char ID,unsigned short Speed)
{
#ifdef __DEBUG__DYNAMIXEL_USE__
	Serial.println("Setting moving speed...");
#endif

	unsigned char PacketMovSpeed[3]={0x20,Speed,Speed>>8};
	unsigned char Buffer[7];
	SendPacket(ID,WRITE,PacketMovSpeed,3);
	ReceivePacket(Buffer,6,TimeOut);
	if(Errors(Buffer))
		return 0;
	else
		return 1;
}

// Set return delay time (0 to 254 where 1 data means 2 s)
bool Dynamixel::SetReturnDelayTime(unsigned char ID, unsigned char Delay)
{
#ifdef __DEBUG__DYNAMIXEL_USE__
	Serial.println("Setting return delay time...");
#endif

	unsigned char PacketReturnDelayTime[2]={0x05,Delay};
	unsigned char Buffer[7];
	SendPacket(ID,WRITE,PacketReturnDelayTime,2);
	ReceivePacket(Buffer,6,TimeOut);
	if(Errors(Buffer))
		return 0;
	else
		return 1;	
}

// Set max torque (0 to 1023 where 1023 means 100% of maximum torque)
bool Dynamixel::SetMaxTorque(unsigned char ID, unsigned short Torque)
{
#ifdef __DEBUG__DYNAMIXEL_USE__
	Serial.println("Setting max torque...");
#endif

	unsigned char PacketMaxTorque[3]={0x0E,Torque,Torque>>8};
	unsigned char Buffer[7];
	SendPacket(ID,WRITE,PacketMaxTorque,3);
	ReceivePacket(Buffer,6,TimeOut);
	if(Errors(Buffer))
		return 0;
	else
		return 1;
}

// Set if torque is enable (0 means no torque and 1 generates torque)
bool Dynamixel::SetTorqueEnable(unsigned char ID, bool Enable)
{
#ifdef __DEBUG__DYNAMIXEL_USE__
	Serial.println("Setting torque enable...");
#endif

	unsigned char PacketTorqueEnable[2]={0x18,Enable};
	unsigned char Buffer[7];
	SendPacket(ID,WRITE,PacketTorqueEnable,2);
	ReceivePacket(Buffer,6,TimeOut);
	if(Errors(Buffer))
		return 0;
	else
		return 1;
}

// Set the CW margin of the compliance (0 to 255, unit is the same as goal pos)
bool Dynamixel::SetCWComplianceMargin(unsigned char ID, unsigned char Margin)
{
#ifdef __DEBUG__DYNAMIXEL_USE__
	Serial.println("Setting CW compliance margin...");
#endif

	unsigned char PacketComplianceMargin[2]={0x1A,Margin};
	unsigned char Buffer[7];
	SendPacket(ID,WRITE,PacketComplianceMargin,2);
	ReceivePacket(Buffer,6,TimeOut);
	if(Errors(Buffer))
		return 0;
	else
		return 1;
}

// Set the CCW margin of the compliance (0 to 255, unit is the same as goal pos)
bool Dynamixel::SetCCWComplianceMargin(unsigned char ID, unsigned char Margin)
{
#ifdef __DEBUG__DYNAMIXEL_USE__
	Serial.println("Setting CCW compliance margin...");
#endif

	unsigned char PacketComplianceMargin[2]={0x1B,Margin};
	unsigned char Buffer[7];
	SendPacket(ID,WRITE,PacketComplianceMargin,2);
	ReceivePacket(Buffer,6,TimeOut);
	if(Errors(Buffer))
		return 0;
	else
		return 1;
}

// Set the CW slope of the compliance (0 to 255 znd 255 is when the most flexibility is obtained)
bool Dynamixel::SetCWComplianceSlope(unsigned char ID, unsigned char Slope)
{
#ifdef __DEBUG__DYNAMIXEL_USE__
	Serial.println("Setting CW compliance slope...");
#endif

	unsigned char PacketComplianceSlope[2]={0x1C,Slope};
	unsigned char Buffer[7];
	SendPacket(ID,WRITE,PacketComplianceSlope,2);
	ReceivePacket(Buffer,6,TimeOut);
	if(Errors(Buffer))
		return 0;
	else
		return 1;
}

// Set the CCW slope of the compliance (0 to 255 and 255 is when the most flexibility is obtained)
bool Dynamixel::SetCCWComplianceSlope(unsigned char ID, unsigned char Slope)
{
#ifdef __DEBUG__DYNAMIXEL_USE__
	Serial.println("Setting CCW compliance slope...");
#endif

	unsigned char PacketComplianceSlope[2]={0x1D,Slope};
	unsigned char Buffer[7];
	SendPacket(ID,WRITE,PacketComplianceSlope,2);
	ReceivePacket(Buffer,6,TimeOut);
	if(Errors(Buffer))
		return 0;
	else
		return 1;
}

// Set the maximum torque limit (0 to 1023 where 1023 means 100% of the maximum torque)
bool Dynamixel::SetTorqueLimit(unsigned char ID, unsigned short Torque)
{
#ifdef __DEBUG__DYNAMIXEL_USE__
	Serial.println("Setting torque limit...");
#endif

	unsigned char PacketTorqueLimit[3]={0x22,Torque,Torque>>8};
	unsigned char Buffer[7];
	SendPacket(ID,WRITE,PacketTorqueLimit,3);
	ReceivePacket(Buffer,6,TimeOut);
	if(Errors(Buffer))
		return 0;
	else
		return 1;
}
								

///////////////////// READ Functions //////////////////////////////


// Get current clockwise angle limit (in degrees)
short Dynamixel::GetCWAngleLimit(unsigned char ID)
{
#ifdef __DEBUG__DYNAMIXEL_USE__
	Serial.println("Getting CW angle limit...");
#endif

	unsigned char Buffer[9];	
	unsigned char PacketAngleLimit[2]={0x06,2};
	SendPacket(ID,READ,PacketAngleLimit,2);
	ReceivePacket(Buffer,8,TimeOut);
	if(!Errors(Buffer))
		return (Buffer[5]+(Buffer[6]<<8)); 
	return(-1);
	
}

// Get current counter clockwise angle limit (in degrees)
short Dynamixel::GetCCWAngleLimit(unsigned char ID)
{
#ifdef __DEBUG__DYNAMIXEL_USE__
	Serial.println("Getting CCW angle limit...");
#endif

	unsigned char Buffer[9];	
	unsigned char PacketAngleLimit[2]={0x08,2};
	SendPacket(ID,READ,PacketAngleLimit,2);
	ReceivePacket(Buffer,8,TimeOut);
	if(!Errors(Buffer))
		return (Buffer[5]+(Buffer[6]<<8));
	return -1;
}

// Return the moving speed
short Dynamixel::GetMovSpeed(unsigned char ID)
{
#ifdef __DEBUG__DYNAMIXEL_USE__
	Serial.println("Getting moving speed...");
#endif

	unsigned char Buffer[9];	
	unsigned char PacketMovSpeed[2]={0x20,2};
	SendPacket(ID,READ,PacketMovSpeed,2);
	ReceivePacket(Buffer,8,TimeOut);
	if(!Errors(Buffer))
		return (Buffer[5]+(Buffer[6]<<8));
	return -1;
}

// Return the return delay time
short Dynamixel::GetReturnDelayTime(unsigned char ID)
{
#ifdef __DEBUG__DYNAMIXEL_USE__
	Serial.println("Getting return delay time...");
#endif

	unsigned char Buffer[8];	
	unsigned char PacketReturnDelayTime[2]={0x05,1};
	SendPacket(ID,READ,PacketReturnDelayTime,2);
	ReceivePacket(Buffer,7,TimeOut);
	if(!Errors(Buffer))
		return (Buffer[5]);
	return -1;
	
}

// Returns the default value of the maximum torque
short Dynamixel::GetMaxTorque(unsigned char ID)
{
#ifdef __DEBUG__DYNAMIXEL_USE__
	Serial.println("Getting max torque...");
#endif

	unsigned char Buffer[9];	
	unsigned char PacketMaxTorque[2]={0x0E,2};
	SendPacket(ID,READ,PacketMaxTorque,2);
	ReceivePacket(Buffer,8,TimeOut);
	if(!Errors(Buffer))
		return (Buffer[5]+(Buffer[6]<<8));
	return -1;
	
}

// Return if torque is enable or disable
char Dynamixel::GetTorqueEnable(unsigned char ID)
{
#ifdef __DEBUG__DYNAMIXEL_USE__
	Serial.println("Getting torque enable...");
#endif

	unsigned char Buffer[8];	
	unsigned char PacketTorqueEnable[2]={0x18,1};
	SendPacket(ID,READ,PacketTorqueEnable,2);
	ReceivePacket(Buffer,7,TimeOut);
	if(!Errors(Buffer))
		return (Buffer[5]);
	return -1;	
}
	
// Return the CW margin of the compliance
short Dynamixel::GetCWComplianceMargin(unsigned char ID)
{
#ifdef __DEBUG__DYNAMIXEL_USE__
	Serial.println("Getting CW compliance margin...");
#endif

	unsigned char Buffer[8];	
	unsigned char PacketComplianceMargin[2]={0x1A,1};
	SendPacket(ID,READ,PacketComplianceMargin,2);
	ReceivePacket(Buffer,7,TimeOut);
	if(!Errors(Buffer))
		return (Buffer[5]);
	return -1;
	
}

// Return the CCW margin of the compliance
short Dynamixel::GetCCWComplianceMargin(unsigned char ID)
{
#ifdef __DEBUG__DYNAMIXEL_USE__
	Serial.println("Getting CCW compliance margin...");
#endif

	unsigned char Buffer[8];	
	unsigned char PacketComplianceMargin[2]={0x1B,1};
	SendPacket(ID,READ,PacketComplianceMargin,2);
	ReceivePacket(Buffer,7,TimeOut);
	if(!Errors(Buffer))
		return (Buffer[5]);
	return -1;	
}
	
// Return the CW slope of the compliance
short Dynamixel::GetCWComplianceSlope(unsigned char ID)
{
#ifdef __DEBUG__DYNAMIXEL_USE__
	Serial.println("Getting CW compliance slope...");
#endif

	unsigned char Buffer[8];	
	unsigned char PacketComplianceSlope[2]={0x1C,1};
	SendPacket(ID,READ,PacketComplianceSlope,2);
	ReceivePacket(Buffer,7,TimeOut);
	if(!Errors(Buffer))
		return (Buffer[5]);
	return -1;	
}

// Return the CCW slope of the compliance
short Dynamixel::GetCCWComplianceSlope(unsigned char ID)
{
#ifdef __DEBUG__DYNAMIXEL_USE__
	Serial.println("Getting CCW compliance slope...");
#endif

	unsigned char Buffer[8];	
	unsigned char PacketComplianceSlope[2]={0x1D,1};
	SendPacket(ID,READ,PacketComplianceSlope,2);
	ReceivePacket(Buffer,7,TimeOut);
	if(!Errors(Buffer))
		return (Buffer[5]);
	return -1;
}

// Return the limit of the maximum torque
short	Dynamixel::GetTorqueLimit(unsigned char ID)
{
#ifdef __DEBUG__DYNAMIXEL_USE__
	Serial.println("Getting torque limit...");
#endif

	unsigned char Buffer[9];	
	unsigned char PacketTorqueLimit[2]={0x22,2};
	SendPacket(ID,READ,PacketTorqueLimit,2);
	ReceivePacket(Buffer,8,TimeOut);
	if(!Errors(Buffer))
		return (Buffer[5]+(Buffer[6]<<8));
	return -1;	
}
	
// Return the present position
short Dynamixel::GetPresentPosition(unsigned char ID)
{
#ifdef __DEBUG__DYNAMIXEL_USE__
	Serial.println("Getting present position...");
#endif

	unsigned char Buffer[9];	
	unsigned char PacketPresentPosition[2]={0x24,2};
	SendPacket(ID,READ,PacketPresentPosition,2);
	ReceivePacket(Buffer,8,TimeOut);
#ifdef __DEBUG__DYNAMIXEL_USE__
	Serial.print("Error :");
	Serial.println(Errors(Buffer));
#endif	
	if(!Errors(Buffer))
		return (Buffer[5]+(Buffer[6]<<8));
	return -1;	
}

// Return the present speed
short Dynamixel::GetPresentSpeed(unsigned char ID)
{
#ifdef __DEBUG__DYNAMIXEL_USE__
	Serial.println("Getting present speed...");
#endif

	unsigned char Buffer[9];	
	unsigned char PacketPresentSpeed[2]={0x26,2};
	SendPacket(ID,READ,PacketPresentSpeed,2);
	ReceivePacket(Buffer,8,TimeOut);
	if(!Errors(Buffer))
	{
		short Speed=Buffer[5]+(Buffer[6]<<8);
		return (Speed&(0<<11));
	}
	return -1;
}


// Return the present load
short Dynamixel::GetPresentLoad(unsigned char ID)
{
#ifdef __DEBUG__DYNAMIXEL_USE__
	Serial.println("Getting present load...");
#endif
	
	unsigned char Buffer[9];	
	unsigned char PacketPresentLoad[2]={0x28,2};
	SendPacket(ID,READ,PacketPresentLoad,2);
	ReceivePacket(Buffer,8,TimeOut);
	if(!Errors(Buffer))
		return (Buffer[5]+(Buffer[6]<<8));
	return -1;	
}


// Return if there is any movement
char Dynamixel::GetMoving(unsigned char ID)
{
#ifdef __DEBUG__DYNAMIXEL_USE__
	Serial.println("Getting moving...");
#endif

	unsigned char Buffer[8];	
	unsigned char PacketMoving[2]={0x2E,1};
	SendPacket(ID,READ,PacketMoving,2);
	ReceivePacket(Buffer,7,TimeOut);
	if(!Errors(Buffer))
		return (Buffer[5]);
	return -1;
}
