/**\file dynamixelUse.h 
 * \brief A class to compose frames to send to servomotors.
 * \details See also : \n \n dynamixelUse.cpp \n The C++ File Reference of this header. \n \n
 * dynamixel.h \n The library use to communicate with servomotors. \n \n dynamixel.cpp \n And his C++ File Reference.*/

#ifndef dynamixelUse_h
#define dynamixelUse_h



#include <WProgram.h>
#include "../dynamixel/dynamixel.h"


//#define __DEBUG__DYNAMIXEL_USE__



#define delete(arg) arg.~Dynamixel()// Permission to call destructor by calling a virtual delete function

/**\class Dynamixel
 * \brief A class to control servomotors dynamixel RX-24F from Robotis via an Arduino Mega 2560 board.\n
 * \details See also : \n \n dynamixelUse.h \n Here is a link to the Header File Reference.\n
 * dynamixelUse.cpp \n Here is a link to the C++ File Reference.\n \n 
 * dynamixel.h \n Here is a link to the library used to communicate with servomotors.\n
 * dynamixel.cpp \n And here the link for his C++ File Reference.*/

class Dynamixel
{
public:
///////////// General functions /////////////

/*! Constructor, initialize communication with servomotors */
					Dynamixel();
/*! Destructor, close communication with servomotors */
					~Dynamixel();

/*! Send a ping to a servomotor 
 * \param ID Defines the ID of the servo to whom send the ping 
 * \return a boolean, if true, it means ping is a success and false, a failure. */
	bool			SendPing(unsigned char ID);

/*! Send a packet manually to a servomotor
 * \param ID Defines the ID of the servo to whom send the packet
 * \param Instruction Which Instruction to execute (see the defines of \link dynamixel.h \endlink to know which instruction can be send)
 * \param Parameters The parameters of the instruction (see <a href=http://support.robotis.com/en/>Robotis documentation</a> for additional information)
 * \param NbParam Just give the number of parameters for the packet
 * \param LengthToRead Finally, the length of data to read (this length can be known easily, in case of WRITE instruction it will always be 0, in case of READ, it depends on how many data you have asked to read
 * \return a boolean, if true, it means it is a success and false, a failure. */
	bool			ManSend(unsigned char ID,unsigned char Instruction,unsigned char *Parameters,int NbParam,int LengthToRead); 

/*! Check if a return packet contains no error
 * \param Packet The packet to be checked
 * \return an int (see \link Errors \endlink from dynamixel.h to know what is returned). */
	int				CheckPacket(unsigned char *Packet);

/*! Empty reception buffer (commonly use after an error occurenced to be sure next frame will not be affected)
 * \return no return. */
	void			ClearReceptionBuffer();

/*! Quit the Alarm Shutdown mode to reuse servomotors normally (Alarm Shutdown mode is activated when an error occurenced on the servomotor)
 * \param ID Defines the ID of the servo to talk with
 * \return a boolean, if true, it means it is a success and false, a failure. */
	bool			LeaveAlarmShutdownMode(unsigned char ID);



///////////// WRITE functions /////////////

/*! Set goal position (0 to 1023 where 0 means max CW angle and 1023 means max CCW angle)
 * \param ID Defines the ID of the servo to talk with
 * \param Angle The angle to reach
 * \return a boolean, if true, it means it is a success and false, a failure. */
	bool			SetGoalPosition(unsigned char ID,unsigned short Angle);

/*! Set CW Angle Limit
 * \param ID Defines the ID of the servo to talk with
 * \param Angle The limit angle to set
 * \return a boolean, if true, it means it is a success and false, a failure. */
	bool			SetCWAngleLimit(unsigned char ID,unsigned short Angle);

/*! Set CCW Angle Limit
 * \param ID Defines the ID of the servo to talk with
 * \param Angle The limit angle to set
 * \return a boolean, if true, it means it is a success and false, a failure. */
	bool			SetCCWAngleLimit(unsigned char ID,unsigned short Angle);
	
/*! Set speed (0 to 1023 where 1023 means about 114 rpm - maximum speed)
 * \param ID Defines the ID of the servo to talk with
 * \param Speed Speed to be defined
 * \return a boolean, if true, it means it is a success and false, a failure. */
 	bool			SetMovSpeed(unsigned char ID,unsigned short Speed);
	
/*! Set return delay time (0 to 254 where 1 data means 2 us)
 * \param ID Defines the ID of the servo to talk with
 * \param Delay As explain in the description of the function, the unit is 2 us so when you put 150 inside it means 300 us of delay
 * \return a boolean, if true, it means it is a success and false, a failure. */
	bool			SetReturnDelayTime(unsigned char ID, unsigned char Delay);

/*! Set the default value of the max torque (0 to 1023 where 1023 means 100% of maximum torque)
 * \param ID Defines the ID of the servo to talk with
 * \param Torque The torque limit to set
 * \return a boolean, if true, it means it is a success and false, a failure. */
	bool			SetMaxTorque(unsigned char ID, unsigned short Torque); 

/*! Set if torque is enable (0 means no torque and 1 generates torque)
 * \param ID Defines the ID of the servo to talk with
 * \param Enable Define if there is torque
 * \return a boolean, if true, it means it is a success and false, a failure. */
	bool			SetTorqueEnable(unsigned char ID, bool Enable);
	
/*! Set the CW margin of the compliance (0 to 255, unit is the same as goal pos)
 * \param ID Defines the ID of the servo to talk with
 * \param Margin The margin to define
 * \return a boolean, if true, it means it is a success and false, a failure. */
	bool			SetCWComplianceMargin(unsigned char ID, unsigned char Margin);

/*! Set the CCW margin of the compliance (0 to 255, unit is the same as goal pos)
 * \param ID Defines the ID of the servo to talk with
 * \param Margin The margin to define
 * \return a boolean, if true, it means it is a success and false, a failure. */ 
	bool			SetCCWComplianceMargin(unsigned char ID, unsigned char Margin);
	
/*! Set the CW slope of the compliance (0 to 255 znd 255 is when the most flexibility is obtained)
 * \param ID Defines the ID of the servo to talk with
 * \param Slope The slope to define
 * \return a boolean, if true, it means it is a success and false, a failure. */
	bool			SetCWComplianceSlope(unsigned char ID, unsigned char Slope);

/*! Set the CCW slope of the compliance (0 to 255 znd 255 is when the most flexibility is obtained)
 * \param ID Defines the ID of the servo to talk with
 * \param Slope The slope to define
 * \return a boolean, if true, it means it is a success and false, a failure. */
	bool			SetCCWComplianceSlope(unsigned char ID, unsigned char Slope); 
	
/*! Set the maximum torque limit (0 to 1023 where 1023 means 100% of the maximum torque)
 * \param ID Defines the ID of the servo to talk with
 * \param Torque The Torque Limit (the default value of the torque limit is define \link SetMaxTorque \endlink)
 * \return a boolean, if true, it means it is a success and false, a failure. */
	bool			SetTorqueLimit(unsigned char ID, unsigned short Torque);
																									

///////////// READ functions /////////////

/*! Get the CW angle limit
 * \param ID Defines the ID of the servo to talk with
 * \return a short, the clockwise angle limit (in degrees) or -1 if an error occurenced.*/
	short			GetCWAngleLimit(unsigned char ID);

/*! Get the CCW angle limit
 * \param ID Defines the ID of the servo to talk with
 * \return a short, the counter clockwise angle limit (in degrees) or -1 if an error occurenced.*/
	short			GetCCWAngleLimit(unsigned char ID);

/*! Get the moving speed
 * \param ID Defines the ID of the servo to talk with
 * \return a short, the speed (0 means 0% and 0xFE means 100% of the maximum speed) or -1 if an error occurenced.*/
	short			GetMovSpeed(unsigned char ID);
	
/*! Get the return delay time (the time between the execution and the return packet)
 * \param ID Defines the ID of the servo to talk with
 * \return a short, the return delay time (0 to 254 where 1 data means 2 s) or -1 if an error occurenced.*/
	short			GetReturnDelayTime(unsigned char ID);

/*! Get the maximum torque i.e the default value of the torque limit(see : \link SetTorqueLimit \endlink and \link GetTorqueLimit \endlink)
 * \param ID Defines the ID of the servo to talk with
 * \return a short, the maximum torque (0 to 1023 where 1023 means 100% of maximum torque) or -1 if an error occurenced.*/
	short			GetMaxTorque(unsigned char ID);

/*! Get if torque is enable or disable
 * \param ID Defines the ID of the servo to talk with
 * \return a char, 1 if torque is enable, 0 if torque is disable or -1 if an error occurenced.*/
	char			GetTorqueEnable(unsigned char ID);
	
/*! Get the CW margin of the compliance
 * \param ID Defines the ID of the servo to talk with
 * \return a short, the clockwise margin of the compliance (same unit as \link SetGoalPosition \endlink) or -1 if an error occurenced.*/
	short			GetCWComplianceMargin(unsigned char ID);

/*! Get the CCW margin of the compliance
 * \param ID Defines the ID of the servo to talk with
 * \return an unsigned char, the counter clockwise margin of the compliance (same unit as \link SetGoalPosition \endlink) or -1 if an error occurenced.*/
	short			GetCCWComplianceMargin(unsigned char ID);

/*! Get the CW margin of the compliance
 * \param ID Defines the ID of the servo to talk with
 * \return a short, the clockwise margin of the compliance (0 to 255 znd 255 is when the most flexibility is obtained) or -1 if an error occurenced.*/
	short			GetCWComplianceSlope(unsigned char ID);

/*! Get the CCW margin of the compliance
 * \param ID Defines the ID of the servo to talk with
 * \return a short, the counter clockwise margin of the compliance (0 to 255 znd 255 is when the most flexibility is obtained) or -1 if an error occurenced.*/
	short			GetCCWComplianceSlope(unsigned char ID);

/*! Get the torque limit (the default value use is the MaxTorque (see : \link SetMaxTorque \endlink and \link GetMaxTorque \endlink).
 * \param ID Defines the ID of the servo to talk with
 * \return a short, the torque limit (0 to 1023 where 1023 means 100% of maximum torque) or -1 if an error occurenced.*/	
	short			GetTorqueLimit(unsigned char ID);
	
/*! Get the actual position
 * \param ID Defines the ID of the servo to talk with
 * \return a short, the present position (same unit as \link SetGoalPosition \endlink) or -1 if an error occurenced.*/	
	short			GetPresentPosition(unsigned char ID);

/*! Get the actual speed
 * \param ID Defines the ID of the servo to talk with
 * \return a short, the present speed (same unit as \link SetMovSpeed \endlink) or -1 if an error occurenced.*/	
	short			GetPresentSpeed(unsigned char ID);

/*! Get the actual load
 * \param ID Defines the ID of the servo to talk with
 * \return a short, the present load (same unit as \link GetTorqueLimit \endlink) or -1 if an error occurenced.*/	
	short			GetPresentLoad(unsigned char ID);

/*! Get if there is any movement on the servomotor
 * \param ID Defines the ID of the servo to talk with
 * \return a char, 1 means that there is no movement, 0 means that an instruction is in progress and -1 that an error occurenced.*/	
	char			GetMoving(unsigned char ID);


///////////// Class variable /////////////
/*! Length of the timeout reception */	
	unsigned long	TimeOut;
	
};


#endif