/**\file motors.cpp 
 * \brief A simple library to send instructions and receive data from motors <em>Faulhaber</em> via RS232 link.
 * \details See also : \n \n motorsControl.h \n The header File Reference.*/

/* Methods for motors control and get information */

#include "motors.h"


////////////////////// General functions /////////////////////

// Initialize link with motors right and left
void InitMotors()
{
  Serial1.begin(115200);                   // For communication with the left motor
  Serial2.begin(115200);                   // For communication with the right motor
}



// Methods to enable left or right
bool EnableLeft()
{
	Serial1.flush();
	Serial1.println("EN");
	return CheckOKLeft();
}

bool EnableRight()
{
	Serial2.flush();
	Serial2.println("EN");
	return CheckOKRight();
}


// Methods to disable left or right
bool DisableLeft()
{
	Serial1.flush();
	Serial1.println("DI");
	return CheckOKLeft();
}

bool DisableRight()
{
	Serial2.flush();
	Serial2.println("DI");
	return CheckOKRight();
}


// Methods to reset left or right
bool ResetLeft()
{
	Serial1.flush();
	Serial1.println("RESET");
	return CheckRESETLeft(1000000);
}

bool ResetRight()
{
	Serial2.flush();
	Serial2.println("RESET");
	return CheckRESETRight(1000000);
}


// Methods to wait for an answer from left or right
char ReadBufferLeft (char *Buffer,int NbMax,long TimeOut_ms)
{
	int i=0;
	bool FlagCR=false;
	long Cpt_us=0;									// Counter for TimeOut

	do {		
		if (Serial1.available()) 
		{
			Buffer[i++]=Serial1.read();
			if (Buffer[i-1]=='\n') FlagCR=true;
			if (i>NbMax) {
				return -1;
			}
		}

		if (Cpt_us>TimeOut_ms) 
		{
			return -2;
		}
		delayMicroseconds (1);
		Cpt_us++;
	}	
	while (FlagCR == false); 
	Buffer[i]=0;
	return 0;	
}

char ReadBufferRight (char *Buffer,int NbMax,long TimeOut_ms)
{
	int i=0;
	bool FlagCR=false;
	long Cpt_us=0;										// Counter for TimeOut
	do {
		if (Serial2.available()) 
		{
			Buffer[i++]=Serial2.read();
			if (Buffer[i-1]=='\n') FlagCR=true;
			if (i>NbMax) {
				return -1;
			}
		}

		if (Cpt_us>TimeOut_ms) 
		{
			return -2;
		}
		delayMicroseconds (1);
		Cpt_us++;
	}
	while (FlagCR == false); 
	Buffer[i]=0;
	return 0;
}

// Methods to check if an "OK" packet has been received
bool CheckOKLeft(long TimeOut_us)
{
#ifdef __DEBUG__MOTORS__
	Serial.println("Waiting for left answer...  ");
#endif
	char Buffer[5]="";
	ReadBufferLeft(Buffer,4,TimeOut_us);
	if(strcmp (Buffer,"OK\r\n")==0)
		return true;
	return false;
}

bool CheckOKRight(long TimeOut_us)
{
#ifdef __DEBUG__MOTORS__
	Serial.println("Waiting for right answer...  ");
#endif
	char Buffer[5]="";
	ReadBufferRight(Buffer,4,TimeOut_us);
	if(strcmp (Buffer,"OK\r\n")==0)
		return true;
	return false;
}

// Methods to check if a "reset" packet has been received
bool CheckRESETLeft(long TimeOut_us)
{
#ifdef __DEBUG__MOTORS__
	Serial.println("Waiting for left reset answer...  ");
#endif
	char Buffer[22]="";
	ReadBufferLeft(Buffer,21,TimeOut_us);
#ifdef __DEBUG__MOTORS__	
	Serial.println(Buffer);
#endif
	if(strcmp (Buffer,"FAULHABER MCDC3003S\r\n")==0)
		return true;
	return false;
}

bool CheckRESETRight(long TimeOut_us)
{
#ifdef __DEBUG__MOTORS__
	Serial.println("Waiting for right reset answer...  ");
#endif
	char Buffer[22]="";
	ReadBufferRight(Buffer,21,TimeOut_us);
	if(strcmp (Buffer,"FAULHABER MCDC3003S\r\n")==0)
		return true;
	return false;
}


/////////////////////// Set functions ////////////////////////


// Methods to set speed left or right
bool SetLeftSpeed(long Vit)
{
	Serial1.flush();
	Serial1.print("V");
	Serial1.println(Vit);
	return CheckOKLeft();
}

bool SetRightSpeed(long Vit)
{
	Serial2.flush();
	Serial2.print("V");
	Serial2.println(Vit);
	return CheckOKRight();
}


// Methods to set absolute position left or right
bool SetLeftAbsolutePos(long Pos)
{
	Serial1.flush();
	Serial1.print("LA");
	Serial1.println(Pos);
	return CheckOKLeft();
}

bool SetRightAbsolutePos(long Pos)
{
	Serial2.flush();
	Serial2.print("LA");
	Serial2.println(Pos);
	return CheckOKRight();
}


// Methods to set relative position left or right
bool SetLeftRelativePos(long Pos)
{
	Serial1.flush();
	Serial1.print("LR");
	Serial1.println(Pos);
	return CheckOKLeft();
}

bool SetRightRelativePos(long  Pos)
{
	Serial2.flush();
	Serial2.print("LR");
	Serial2.println(Pos);
	return CheckOKRight();
}


// Methods to set acceleration left or right
bool SetLeftAcc(int Acc)
{
	Serial1.flush();
	Serial1.print("AC");
	Serial1.println(Acc);
	return CheckOKLeft();
}

bool SetRightAcc(int Acc)
{
	Serial2.flush();
	Serial2.print("AC");
	Serial2.println(Acc);
	return CheckOKRight();
}

// Methods to set deceleration left or right
bool SetLeftDec(int Dec)
{
	Serial1.flush();
	Serial1.print("DEC");
	Serial1.println(Dec);
	return CheckOKLeft();
}

bool SetRightDec(int Dec)
{
	Serial2.flush();
	Serial2.print("DEC");
	Serial2.println(Dec);
	return CheckOKRight();
}

// Methods to set peak current limit left or right
bool SetLeftPeakCurrentLimit(int Current)
{
	Serial1.flush();
	Serial1.print("LPC");
	Serial1.println(Current);
	return CheckOKLeft();
}

bool SetRightPeakCurrentLimit(int Current)
{
	Serial2.flush();
	Serial2.print("LPC");
	Serial2.println(Current);
	return CheckOKRight();
}

// Methods to set continuous current limit left or right
bool SetLeftContCurrentLimit(int Current)
{
	Serial1.flush();
	Serial1.print("LCC");
	Serial1.println(Current);
	return CheckOKLeft();
}

bool SetRightContCurrentLimit(int Current)
{
	Serial2.flush();
	Serial2.print("LCC");
	Serial2.println(Current);
	return CheckOKRight();
}

// Methods to notify when position is reached
bool SetLeftNotifyPosition (long Pos)
{
	Serial1.flush();
	if(Pos==0)
		Serial1.println("NP");
	else
	{
		Serial1.print("NP");
		Serial1.println(Pos);
	}
	return CheckOKLeft();
}

bool SetRightNotifyPosition (long Pos)
{
	Serial2.flush();
	if(Pos==0)
		Serial2.println("NP");
	else
	{
		Serial2.print("NP");
		Serial2.println(Pos);
	}
	return CheckOKRight();
}

// Methods to end notify position
bool SetLeftNotifyPositionOff ()
{
	Serial1.flush();
	Serial1.println("NPOFF");
	return CheckOKLeft();
}

bool SetRightNotifyPositionOff ()
{
	Serial2.flush();
	Serial2.println("NPOFF");
	return CheckOKRight();
}

// Methods to notify when velocity is reached
bool SetLeftNotifyVelocity (long Vel)
{
	Serial1.flush();
	Serial1.print("NV");
	Serial1.println(Vel);
	return CheckOKLeft();
}

bool SetRightNotifyVelocity (long Vel)
{
	Serial2.flush();
	Serial2.print("NV");
	Serial2.println(Vel);
	return CheckOKRight();
}

// Methods to end notify velocity
bool SetLeftNotifyVelocityOff ()
{
	Serial1.flush();
	Serial1.println("NVOFF");
	return CheckOKLeft();
}

bool SetRightNotifyVelocityOff ()
{
	Serial2.flush();
	Serial2.println("NVOFF");
	return CheckOKRight();
}

// Methods to move to last position load
bool MovingLeft()
{
	Serial1.flush();
	Serial1.println("M");
	return CheckOKLeft();
}

bool MovingRight()
{
	Serial2.flush();
	Serial2.println("M");
	return CheckOKRight();
}
////////////////////// Get functions /////////////////////////


// Methods to get speed left or right
char GetLeftSpeed(long *Speed)
{
	Serial1.flush();
	Serial1.println("GV");
	char Buffer[10];
	char Ret=ReadBufferLeft(Buffer,9,100000);
	if (Ret!=0) return Ret;

	*Speed=atol(Buffer); 
	return 0;
}

char GetRightSpeed(long *Speed)
{
	Serial2.flush();
	Serial2.println("GV");
	char Buffer[10];
	char Ret=ReadBufferRight(Buffer,9,100000);
	if (Ret!=0) return Ret;

	*Speed=atol(Buffer); 
	return 0;
}

// Methods to get current position left or right
char GetLeftPos(long *Pos)
{
	Serial1.flush();
	Serial1.println("POS");
	char Buffer[10];
	char Ret=ReadBufferLeft(Buffer,9,100000);
	if (Ret!=0) return Ret;

	*Pos=atol(Buffer); 
	return 1;
}

char GetRightPos(long *Pos)
{
	Serial2.flush();
	Serial2.println("POS");
	char Buffer[10];
	char Ret=ReadBufferRight(Buffer,9,100000);
	if (Ret!=0) return Ret;

	*Pos=atol(Buffer); 
	return 1;
}

// Methods to get current acceleration left or right
char GetLeftAcc(int *Acc)
{
	Serial1.flush();
	Serial1.println("GAC");
	char Buffer[10];
	char Ret=ReadBufferLeft(Buffer,9,100000);
	if (Ret!=0) return Ret;

	*Acc=atoi(Buffer); 
	return 0;
}

char GetRightAcc(int *Acc)
{
	Serial2.flush();
	Serial2.println("GAC");
	char Buffer[10];
	char Ret=ReadBufferRight(Buffer,9,100000);
	if (Ret!=0) return Ret;

	*Acc=atoi(Buffer); 
	return 0;
}

// Methods to get current deceleration left or right
char GetLeftDec(int *Dec)
{
	Serial1.flush();
	Serial1.println("GDEC");
	char Buffer[10];
	char Ret=ReadBufferLeft(Buffer,9,100000);
	if (Ret!=0) return Ret;

	*Dec=atoi(Buffer); 
	return 0;
}

char GetRightDec(int *Dec)
{
	Serial2.flush();
	Serial2.println("GDEC");
	char Buffer[10];
	char Ret=ReadBufferRight(Buffer,9,100000);
	if (Ret!=0) return Ret;

	*Dec=atoi(Buffer); 
	return 0;
}

// Methods to get current peak current limit left or right
char GetLeftPeakCurrentLimit(int *Current)
{
	Serial1.flush();
	Serial1.println("GPC");
	char Buffer[10];
	char Ret=ReadBufferLeft(Buffer,9,100000);
	if (Ret!=0) return Ret;

	*Current=atoi(Buffer); 
	return 0;
}

char GetRightPeakCurrentLimit(int *Current)
{
	Serial2.flush();
	Serial2.println("GPC");
	char Buffer[10];
	char Ret=ReadBufferRight(Buffer,9,100000);
	if (Ret!=0) return Ret;

	*Current=atoi(Buffer); 
	return 0;
}

// Methods to get current continuous current limit left or right
char GetLeftContCurrentLimit(int *Current)
{
	Serial1.flush();
	Serial1.println("GCC");
	char Buffer[10];
	char Ret=ReadBufferLeft(Buffer,9,100000);
	if (Ret!=0) return Ret;

	*Current=atoi(Buffer); 
	return 0;
}

char GetRightContCurrentLimit(int *Current)
{
	Serial2.flush();
	Serial2.println("GCC");
	char Buffer[10];
	char Ret=ReadBufferRight(Buffer,9,100000);
	if (Ret!=0) return Ret;

	*Current=atoi(Buffer); 
	return 0;
}

// Methods to get when position is reached
bool GetLeftNotifyPosition()
{
	char Buffer[5];
	ReadBufferLeft(Buffer,4,100000);
	if(strcmp (Buffer,"p\r\n")==0)
		return true;
	return false;
}

bool GetRightNotifyPosition()
{
	char Buffer[5];
	ReadBufferRight(Buffer,4,100000);
	if(strcmp (Buffer,"p\r\n")==0)
		return true;
	return false;
}

// Methods to get when velocity is reached
bool GetLeftNotifyVelocity()
{
	char Buffer[5];
	ReadBufferLeft(Buffer,4,100000);
	if(strcmp (Buffer,"v\r\n")==0)
		return true;
	return false;
}

bool GetRightNotifyVelocity()
{
	char Buffer[5];
	ReadBufferRight(Buffer,4,100000);
	if(strcmp (Buffer,"v\r\n")==0)
		return true;
	return false;
}