/**\file motors.h 
 * \brief A simple library to send instructions and receive data from motors <em>Faulhaber</em> via RS232 link.
 * \details motors.cpp \n Here is the C++ File Reference of this header.*/

#ifndef motors_h
#define motors_h

#include <WProgram.h>



/*
  motors.h - Library for controling motors
  from Faulhaber (with RS232 link)


  Copyright (c) 2011 David Drissi.  All right reserved.
  Property of the University of Angers.

  Contact : philippe.lucidarme@univ-angers.fr

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.  
*/

// #define __DEBUG__MOTORS__


//////////// General functions /////////////////////

// Initialize link with motors right and left
void InitMotors();

// Enable left or right
/*! Enable left motor.
 * \return a boolean, 1 means instruction is done and 0 that instruction has failed.*/
	bool		EnableLeft();
/*! Enable right motor.
 * \return a boolean, 1 means instruction is done and 0 that instruction has failed.*/
	bool		EnableRight();


// Disable left or right
/*! Disable left motor.
 * \return a boolean, 1 means instruction is done and 0 that instruction has failed.*/
	bool		DisableLeft();
/*! Disable right motor.
 * \return a boolean, 1 means instruction is done and 0 that instruction has failed.*/
	bool		DisableRight();


// Reset left or right
/*! Reset left motor.
 * \return a boolean, 1 means instruction is done and 0 that instruction has failed. */
	bool		ResetLeft();
/*! Reset right motor.
 * \return a boolean, 1 means instruction is done and 0 that instruction has failed. */
	bool		ResetRight();

// Read reception buffer left or right
/*! Read left buffer.
 * \param Buffer To save the reception
 * \param NbMax Set the maximum number of characters to read
 * \param TimeOut Set the length of the time out
 * \return a char, 0 means instruction is done, -1 that reception buffer is full and -2 that time out has occured. */
	char		ReadBufferLeft (char *Buffer,int NbMax,long TimeOut_ms);
/*! Read right buffer.
 * \param Buffer To save the reception
 * \param NbMax Set the maximum number of characters to read
 * \param TimeOut Set the length of the time out
 * \return a char, 0 means instruction is done, -1 that reception buffer is full and -2 that time out has occured. */
	char		ReadBufferRight (char *Buffer,int NbMax,long TimeOut_ms);

// Read reception and check if a "ok" is received
/*! Check if a "ok" has been received on left buffer.
 * \param TimeOut_us Set the length of the time out (by default : 100 ms)
 * \return a boolean, 1 means the answer was good and 0 that answer was bad or reception failed. */
	bool		CheckOKLeft(long TimeOut_us=100000);
/*! Check if a "ok" has been received on right buffer.
 * \param TimeOut_us Set the length of the time out (by default : 100 ms)
 * \return a boolean, 1 means the answer was good and 0 that answer was bad or reception failed. */
	bool		CheckOKRight(long TimeOut_us=100000);

// Read reception and check if a "reset" is received
/*! Check if a "reset" has been received on left buffer.
 * \param TimeOut_us Set the length of the time out (by default : 100 ms)
 * \return a boolean, 1 means the answer was good and 0 that answer was bad or reception failed. */
	bool		CheckRESETLeft(long TimeOut_us=100000);
/*! Check if a "reset" has been received on right buffer.
 * \param TimeOut_us Set the length of the time out (by default : 100 ms)
 * \return a boolean, 1 means the answer was good and 0 that answer was bad or reception failed. */
	bool		CheckRESETRight(long TimeOut_us=100000);

//////////// Set functions ////////////////////////

// Set speed left or right (0 to 32767 rpm)
/*! Set left speed (from 0 to 32727 rpm).
 * \param Vit Defines the speed to use
 * \return a boolean, 1 means instruction is done and 0 that instruction has failed.*/
	bool		SetLeftSpeed(long Vit);
/*! Set right speed (from 0 to 32727 rpm).
 * \param Vit Defines the speed to use
 * \return a boolean, 1 means instruction is done and 0 that instruction has failed.*/
	bool		SetRightSpeed(long Vit);


// Set absolute position left or right (-1.8*10^9 to +1.8*10^9)
/*! Set left absolute position (from -1.8*10^9 to +1.8*10^9).
 * \param Pos Defines the position to use
 * \return a boolean, 1 means instruction is done and 0 that instruction has failed.*/
	bool		SetLeftAbsolutePos(long Pos);
/*! Set right absolute position (from -1.8*10^9 to +1.8*10^9).
 * \param Pos Defines the position to use
 * \return a boolean, 1 means instruction is done and 0 that instruction has failed.*/
	bool		SetRightAbsolutePos(long Pos);


// Set relative position left or right (-2.14*10^9 to +2.14*10^9)
/*! Set left relative position (from -2.14*10^9 to +2.14*10^9).
 * \param Pos Defines the position to use
 * \return a boolean, 1 means instruction is done and 0 that instruction has failed.*/
	bool		SetLeftRelativePos(long Pos);
/*! Set right relative position (from -2.14*10^9 to +2.14*10^9).
 * \param Pos Defines the position to use
 * \return a boolean, 1 means instruction is done and 0 that instruction has failed.*/
	bool		SetRightRelativePos(long Pos);

		
// Set acceleration left or right (0 to 30000 r/s^2)
/*! Set left acceleration (from 0 to 30000 r/s^2).
 * \param Acc Defines the acceleration to use
 * \return a boolean, 1 means instruction is done and 0 that instruction has failed.*/
	bool		SetLeftAcc(int Acc);
/*! Set right acceleration (from 0 to 30000 r/s^2).
 * \param Acc Defines the acceleration to use
 * \return a boolean, 1 means instruction is done and 0 that instruction has failed.*/	
	bool		SetRightAcc(int Acc);


// Set deceleration left or right (0 to 30000 r/s^2)
/*! Set left deceleration (from 0 to 30000 r/s^2).
 * \param Dec Defines the deceleration to use
 * \return a boolean, 1 means instruction is done and 0 that instruction has failed.*/
	bool		SetLeftDec(int Dec);
/*! Set right deceleration (from 0 to 30000 r/s^2).
 * \param Dec Defines the deceleration to use
 * \return a boolean, 1 means instruction is done and 0 that instruction has failed.*/
	bool		SetRightDec(int Dec);


// Set peak current left or right (0 to 12000 mA)
/*! Set left peak current limit (from 0 to 12000 mA).
 * \param Current Defines the current limit to use
 * \return a boolean, 1 means instruction is done and 0 that instruction has failed.*/
	bool		SetLeftPeakCurrentLimit(int Current);
/*! Set right peak current limit (from 0 to 12000 mA).
 * \param Current Defines the current limit to use
 * \return a boolean, 1 means instruction is done and 0 that instruction has failed.*/
	bool		SetRightPeakCurrentLimit(int Current);

// Set continuous current limit (0 to 12000 mA)
/*! Set left continuous current limit (from 0 to 12000 mA).
 * \param Current Defines the current limit to use
 * \return a boolean, 1 means instruction is done and 0 that instruction has failed.*/
	bool		SetLeftContCurrentLimit(int Current);
/*! Set right continuous current limit (from 0 to 12000 mA).
 * \param Current Defines the current limit to use
 * \return a boolean, 1 means instruction is done and 0 that instruction has failed.*/
	bool		SetRightContCurrentLimit(int Current);

// Set if motors tell when position is reached
/*! Set left notify position (when specified position is reached send a "notify position" that could be read by \link GetLeftNotifyPosition \endlink.
 * \param Pos Defines the position to reach or pass to send the packet or if 0 or nothing is given the packet will be send when nominal position will be reached.
 * \return a boolean, 1 means instruction is done and 0 that instruction has failed.*/
	bool		SetLeftNotifyPosition(long Pos=0);
/*! Set right notify position (when specified position is reached send a "notify position" that could be read by \link GetRightNotifyPosition \endlink.
 * \param Pos Defines the position to reach or pass to send the packet or if 0 or nothing is given the packet will be send when nominal position will be reached.
 * \return a boolean, 1 means instruction is done and 0 that instruction has failed.*/
	bool		SetRightNotifyPosition(long Pos=0);

// Desactivate the notify position mode
/*! Stop sending a "notify position" packet when left position is reached.
 * \return a boolean, 1 means instruction is done and 0 that instruction has failed.*/
	bool		SetLeftNotifyPositionOff();
/*! Stop sending a "notify position" packet when right position is reached.
 * \return a boolean, 1 means instruction is done and 0 that instruction has failed.*/
	bool		SetRightNotifyPositionOff();

// Set if motors tell when velocity is reached
/*! Set left notify velocity (when specified velocity is reached send a "notify position" that could be read by \link GetLeftNotifyVelocity \endlink.
 * \param Vel Defines the velocity to reach or pass to send the packet.
 * \return a boolean, 1 means instruction is done and 0 that instruction has failed.*/
	bool		SetLeftNotifyVelocity(long Vel);
/*! Set right notify velocity (when specified velocity is reached send a "notify position" that could be read by \link GetRightNotifyVelocity \endlink.
 * \param Vel Defines the velocity to reach or pass to send the packet.
 * \return a boolean, 1 means instruction is done and 0 that instruction has failed.*/
	bool		SetRightNotifyVelocity(long Vel);

// Desactivate the notify velocity mode
/*! Stop sending a "notify velocity" packet when left velocity is reached.
 * \return a boolean, 1 means instruction is done and 0 that instruction has failed.*/
	bool		SetLeftNotifyVelocityOff();
/*! Stop sending a "notify velocity" packet when right velocity is reached.
 * \return a boolean, 1 means instruction is done and 0 that instruction has failed.*/
	bool		SetRightNotifyVelocityOff();
	
// Move to reach position load previously
/*! Start moving to the last left position load.
 * \return a boolean, 1 means instruction is done and 0 that instruction has failed.*/
	bool		MovingLeft();
/*! Start moving to the last right position load.
 * \return a boolean, 1 means instruction is done and 0 that instruction has failed.*/
	bool		MovingRight();

//////////// Get functions /////////////////////////

// Get speed left or right (0 to 32767 rpm)
/*! Set left speed (from 0 to 32767 rpm).
 * \return a long, the speed in rpm.*/	
	char		GetLeftSpeed(long *Speed);
/*! Set right speed (from 0 to 32767 rpm). 
 * \return a long, the speed in rpm.*/
	char		GetRightSpeed(long *Speed);

	
// Get position left or right (-1.8*10^9 to +1.8*10^9)
/*! Get left position (from -1.8*10^9 to +1.8*10^9).
 * \return 1 in case of success negative value if not.*/
 	char		GetLeftPos(long *Pos);
/*! Get right position (from -1.8*10^9 to +1.8*10^9).
 * \return 1 in case of success negative value if not.*/
	char		GetRightPos(long *Pos);


// Get acceleration left or right (0 to 30000 r/s^2)
/*! Get left acceleration (from 0 to 30000 r/s^2).
 * \return an int, the acceleration in r/s^2.*/
	char		GetLeftAcc(int *Acc);
/*! Get right acceleration (from 0 to 30000 r/s^2).
 * \return an int, the acceleration in r/s^2.*/
	char		GetRightAcc(int *Acc);


// Get deceleration left or right (0 to 30000 r/s^2)
/*! Get left deceleration (from 0 to 30000 r/s^2).
 * \return an int, the deceleration in r/s^2.*/
	char		GetLeftDec(int *Dec);
/*! Get right deceleration (from 0 to 30000 r/s^2).
 * \return an int, the deceleration in r/s^2.*/
	char		GetRightDec(int *Dec);


// Get peak current limit left or right (0 to 12000 mA)
/*! Get left peak current limit (from 0 to 12000 mA).
 * \return an int, the current limit.*/
	char		GetLeftPeakCurrentLimit(int *Current);
/*! Get right peak current limit (from 0 to 12000 mA).
 * \return an int, the current limit.*/
	char		GetRightPeakCurrentLimit(int *Current);


// Get continuous current limit left or right (0 to 12000 mA)
/*! Get left continuous current limit (from 0 to 12000 mA).
 * \return an int, the current limit.*/
	char		GetLeftContCurrentLimit(int *Current);
/*! Get right continuous current limit (from 0 to 12000 mA).
 * \return an int, the current limit.*/
	char		GetRightContCurrentLimit(int *Current);

// Check if a "notify position" packet is arrived
/*! Read left buffer to see if a "notify position" packet has been received (Check \link SetLeftNotifyPosition \endlink for more information about this function).
 * \return a boolean, 1 means instruction is done and 0 that instruction has failed.*/
	bool		GetLeftNotifyPosition();
/*! Read right buffer to see if a "notify position" packet has been received (Check \link SetRightNotifyPosition \endlink for more information about this function).
 * \return a boolean, 1 means instruction is done and 0 that instruction has failed.*/
	bool		GetRightNotifyPosition();

// Check if a "notify velocity" packet is arrived
/*! Read left buffer to see if a "notify velocity" packet has been received (Check \link SetLeftNotifyVelocity \endlink for more information about this function).
 * \return a boolean, 1 means instruction is done and 0 that instruction has failed.*/
	bool		GetLeftNotifyVelocity();
/*! Read right buffer to see if a "notify velocity" packet has been received (Check \link SetRightNotifyVelocity \endlink for more information about this function).
 * \return a boolean, 1 means instruction is done and 0 that instruction has failed.*/
	bool		GetRightNotifyVelocity();


#endif