/**\file uSonic.cpp 
 * \brief A function that receives distances returned  by the ultrasonic sensors.
 * \details See also : \n \n uSonic.h \n The header File Reference.*/

/* Get the value return by each ultrasonic sensor */

#include "uSonic.h"



// US_Init
// Initialize IC line
void US_Init()
{
	Wire.begin();
}


// US_RequestMesurement
// Send the command for requesting a mesurement in centimeters
// Parameters : Adress = adress of the sensor 
// Return :		0 = success
//				-1 = failled to send command
char US_RequestMesurement(byte Adress)
{
	Wire.beginTransmission(Adress>>1);             		// Start communticating with SRF235
	Wire.send(0x00);                             		// Send Command Byte
	Wire.send(0x51);                                	// Send 0x51 to start a ranging
#ifdef __DEBUG__US__
	Serial.println("Finalize transmission...");
#endif	
	if (Wire.endTransmission()!=0)
	{
		return -1;	
#ifdef __DEBUG__US__
	Serial.println("Transmission failled");
#endif	
	}
#ifdef __DEBUG__US__
	Serial.println("Transmission success");
#endif	

	return 0;
}



// US_GetRange
// Get distance mesured by the sensor in centimeters
// US_RequestMesurement must has be sent previously (wait 70ms before reading result)
// Parameters : Adress = adress of the sensor 
// return :		>0 : Distance in centimeters
//				-1 : failled to send data to the module
//				-2 : failled to receive data from the module
int US_GetRange(byte Adress)								
{ 
#ifdef __DEBUG__US__
	Serial.println("Getting US sensor 1 value...");
#endif

	int Range; 
	byte HighByte;										// Stores high byte from ranging
	byte LowByte;										// Stored low byte from ranging
  
	Wire.beginTransmission(Adress>>1);             		// Start communicating with SRFmodule
	Wire.send(0x02);                           	// Call the register for start of ranging data
	if (Wire.endTransmission()!=0) return -1;
  
	Wire.requestFrom(Adress>>1, 2);                		// Request 2 bytes from SRF module
	if(Wire.available()== 2)                    		// Wait for data to arrive
	{
		HighByte = Wire.receive();                      // Get high byte
		LowByte = Wire.receive();                       // Get low byte
		return (HighByte << 8) + LowByte;              	// Put them together
	}
	return -2;
}


// US_GetVersion
// Read and return the version number of the sensor with located at adress
// return : 	>=0 version number
//				-1 : failled to send command
//				-2 : data not received successfully
int 	US_GetVersion(byte Adress)
{
#ifdef __DEBUG__US__
	Serial.println("Getting US sensor version number...");
#endif

	Wire.beginTransmission(Adress>>1);             		// Start communicating with SRFmodule
	Wire.send(0x5D);                           			// Call the register 0x5D (version byte)
	if (Wire.endTransmission()!=0)
	return -1;											// return -1 if command is not sent successfully
  
	Wire.requestFrom(Adress>>1, 1);                		// Request 1 bytes from SRF module
	if(Wire.available()== 1)                    		// Wait for data to arrive
		return(Wire.receive());							// Returns version byte
	return -2;											// Data has not been received from the module
}



// US_GetSoftwareRevision
// Read and return the software revision of the sensor with located at adress
// return : 	>=0 Software revision
//				-1 : failled to send command
//				-2 : data not received successfully
int US_GetSoftwareRevision(byte Adress)
{

	Wire.beginTransmission(Adress>>1);             		// Start communicating with SRFmodule
	Wire.send(0x00);                           			// Call the register for software revision
	if (Wire.endTransmission()!=0) return -1;			// Return -1 in case of failure
	
	Wire.requestFrom(Adress>>1, 1);                		// Request 1 bytes from SRF module
	if(Wire.available()== 1)                    		// Wait for data to arrive
		return(Wire.receive());							// Returns version byte
	return -2;	
} 

// US_ChangeAdress
// Change IC adress of one module
// ONLY ONE MODULE MUST BE CONNECTED TO THE LINE
// Old is the current adress of the module
// New is the new adress of the module after the change
// Possible adresses for SFR02 are 0xE0, 0xE2, 0xE4, 0xE6, 0xE8, 0xEA, 0xEC, 0xEE, 0xF0, 0xF2, 0xF4, 0xF6, 0xF8, 0xFA, 0xFC, 0xFE
// Return :  0 success
//           -1 if first frame is not successfully sent
//           -2 if second frame is not successfully sent
//           -3 if third frame is not successfully sent
//           -4 if fourth frame is not successfully sent
int US_ChangeAdress(byte Old,byte New)
{
  Wire.beginTransmission(Old>>1);              // Send first command
  Wire.send(0x00);
  Wire.send(0xA0);  
  if (Wire.endTransmission()!=0) return -1;
 
  Wire.beginTransmission(Old>>1);              // Send second command
  Wire.send(0x00);
  Wire.send(0xAA);  
  if (Wire.endTransmission()!=0) return -2;
 
  Wire.beginTransmission(Old>>1);              // Send third command
  Wire.send(0x00);
  Wire.send(0xA5);  
  if (Wire.endTransmission()!=0) return -3;
 
  Wire.beginTransmission(Old>>1);              // Send fourth command
  Wire.send(0x00);
  Wire.send(New);  
  if (Wire.endTransmission()!=0) return -4;
 
  return 0;                                    // Operation successfull
}

